/**************************************************************************
*
* Filename:    PS5000Acon.c
*
* Copyright:   Pico Technology Limited 2010
*
* Author:      HSM
*
* Description:
*   This is a console mode program that demonstrates how to use the
*   PicoScope 5000a series API.
*

*
***************************************************************************/
#include <stdio.h>

/* Headers for Windows */
#ifdef _WIN32
#include "windows.h"
#include <conio.h>
#include "ps5000aApi.h"
#else
#include <sys/types.h>
#include <string.h>

#include <libps5000a-1.0/ps5000aApi.h>
#include "linux_utils.h"
#endif

#define PREF4 __stdcall

int cycles = 0;

#define BUFFER_SIZE 	1024

#define QUAD_SCOPE		4
#define DUAL_SCOPE		2


typedef enum
{
	ANALOGUE,
	DIGITAL,
	AGGREGATED,
	MIXED
}MODE;


typedef struct
{
	short DCcoupled;
	short range;
	short enabled;
}CHANNEL_SETTINGS;

typedef enum
{
	MODEL_NONE = 0,
	MODEL_PS5242A	= 0xA242,
	MODEL_PS5242B	= 0xB242,
	MODEL_PS5243A	= 0xA243,
	MODEL_PS5243B	= 0xB243,
	MODEL_PS5244A	= 0xA244,
	MODEL_PS5244B	= 0xB244,
	MODEL_PS5442A	= 0xA442,
	MODEL_PS5442B	= 0xB442,
	MODEL_PS5443A	= 0xA443,
	MODEL_PS5443B	= 0xB443,
	MODEL_PS5444A	= 0xA444,
	MODEL_PS5444B	= 0xB444
	
} MODEL_TYPE;

typedef enum
{
	SIGGEN_NONE = 0,
	SIGGEN_FUNCTGEN = 1,
	SIGGEN_AWG = 2
} SIGGEN_TYPE;

typedef struct tTriggerDirections
{
	PS5000A_THRESHOLD_DIRECTION channelA;
	PS5000A_THRESHOLD_DIRECTION channelB;
	PS5000A_THRESHOLD_DIRECTION channelC;
	PS5000A_THRESHOLD_DIRECTION channelD;
	PS5000A_THRESHOLD_DIRECTION ext;
	PS5000A_THRESHOLD_DIRECTION aux;
}TRIGGER_DIRECTIONS;

typedef struct tPwq
{
	PS5000A_PWQ_CONDITIONS * conditions;
	short nConditions;
	PS5000A_THRESHOLD_DIRECTION direction;
	unsigned long lower;
	unsigned long upper;
	PS5000A_PULSE_WIDTH_TYPE type;
}PWQ;

typedef struct
{
	short handle;
	MODEL_TYPE				model;
	PS5000A_RANGE			firstRange;
	PS5000A_RANGE			lastRange;
	short					channelCount;
	short					maxValue;
	short					sigGen;
	short					ETS;
	short					AWGFileSize;
	CHANNEL_SETTINGS		channelSettings [PS5000A_MAX_CHANNELS];
	short					digitalPorts;
}UNIT;

unsigned long timebase = 8;
short       oversample = 1;
BOOL      scaleVoltages = TRUE;

unsigned short inputRanges [PS5000A_MAX_RANGES] = {	10, 
	20,
	50,
	100,
	200,
	500,
	1000,
	2000,
	5000,
	10000,
	20000,
	50000};


typedef struct tBufferInfo
{
	UNIT * unit;
	MODE mode;
	short **driverBuffers;
	short **appBuffers;
	short **driverDigBuffers;
	short **appDigBuffers;

} BUFFER_INFO;


/****************************************************************************
* Initialise unit' structure with Variant specific defaults
****************************************************************************/
void get_info(UNIT * unit)
{
	char description [6][25]= { "Driver Version",
		"USB Version",
		"Hardware Version",
		"Variant Info",
		"Serial",
		"Cal Date" };
	short i, r = 0;
	char line [80];
	int variant;
	PICO_STATUS status = PICO_OK;

	if (unit->handle) 
	{
		for (i = 0; i < 6; i++) 
		{
			status = ps5000aGetUnitInfo(unit->handle, line, sizeof (line), &r, i);
			
			printf("%s: %s\n", description[i], line);
		}

		
	}
}



/****************************************************************************
* OpenDevice 
* Parameters 
* - unit        pointer to the UNIT structure, where the handle will be stored
*
* Returns
* - PICO_STATUS to indicate success, or if an error occurred
***************************************************************************/
PICO_STATUS OpenDevice(UNIT *unit)
{
	char ch;
	short value = 0;
	int i;
	struct tPwq pulseWidth;
	struct tTriggerDirections directions;

	LARGE_INTEGER frequency;
	LARGE_INTEGER time1;
	LARGE_INTEGER time2;

	double elapsedTime = 0.000;
	PICO_STATUS status;
	
	// Ticks
	QueryPerformanceFrequency(&frequency);

    // start timer
    QueryPerformanceCounter(&time1);

	status = ps5000aOpenUnit(&(unit->handle), NULL, PS5000A_DR_8BIT);

	QueryPerformanceCounter(&time2);

	elapsedTime = (time2.QuadPart - time1.QuadPart) / frequency.QuadPart; // seconds

	printf("Elapsed time to call ps5000aOpenUnit: %.4f\n", elapsedTime);

	if (status == PICO_POWER_SUPPLY_NOT_CONNECTED)
	{
		status = ps5000aChangePowerSource(unit->handle, PICO_POWER_SUPPLY_NOT_CONNECTED);
	}

	printf("Handle: %d\n", unit->handle);
	
	if (status != PICO_OK) 
	{
		printf("Unable to open device\n");
		printf("Error code : 0x%08lx\n", status);
		while(!_kbhit());
		exit(99); // exit program
	}

	printf("Device opened successfully, cycle %d\n\n", ++cycles);

	// setup devices
	get_info(unit);

	//timebase = 1;

	//PS5000AMaximumValue(unit->handle, &value);
	//unit->maxValue = value;

	//for ( i = 0; i < PS5000A_MAX_CHANNELS; i++) 
	//{
	//	unit->channelSettings[i].enabled = TRUE;
	//	unit->channelSettings[i].DCcoupled = TRUE;
	//	unit->channelSettings[i].range = PS5000A_5V;
	//}

	//memset(&directions, 0, sizeof(struct tTriggerDirections));
	//memset(&pulseWidth, 0, sizeof(struct tPwq));

	//SetDefaults(unit);

	///* Trigger disabled	*/
	//SetTrigger(unit, NULL, 0, NULL, 0, &directions, &pulseWidth, 0, 0, 0, 0, 0);

	return status;
}



/****************************************************************************
* CloseDevice 
****************************************************************************/
void CloseDevice(UNIT *unit)
{
	ps5000aCloseUnit(unit->handle);
}




/****************************************************************************
* main
* 
***************************************************************************/
int main(void)
{
	char ch;
	PICO_STATUS status;
	UNIT unit;

	printf("PS5000A driver example program\n");
	printf("\n\nOpening the device...\n");

	status = OpenDevice(&unit);


	ch = '.';
	while (ch != 'X')
	{
		//DisplaySettings(&unit);

		printf("\n\n");
		
		printf("X - Exit\n");
		printf("Operation:");

		ch = toupper(_getch());

		printf("\n\n");
		switch (ch) 
		{
			case 'X':
				break;

			default:
				printf("Invalid operation\n");
				break;
		}
	}
	CloseDevice(&unit);

	return 1;
}
